@@ -14,36 +14,6 @@ class Collate:
     ANNOTATIONS = {'blast':'*','partial':'^','exact':''}
     REFGENES = pathlib.Path(__file__).parent / "db" / "refgenes_latest.csv"
     MATCH = ["ALLELEX", "BLASTX", "EXACTX", "POINTX"]
-    NONRTM = [
-        "Amikacin/Gentamicin/Kanamycin/Tobramycin",
-        "Amikacin/Kanamycin",
-        "Amikacin/Kanamycin/Tobramycin",
-        "Amikacin/Quinolone",
-        "Amikacin/Tobramycin",
-        "Aminoglycosides",
-        "Gentamicin",
-        "Gentamicin/Kanamycin/Tobramycin",
-        "Gentamicin/Tobramcyin",
-        "Kanamycin",
-        "Kanamycin/Tobramycin",
-        "Spectinomycin",
-        "Streptogramin",
-        "Streptomycin",
-        "Streptomycin/Spectinomycin",
-        "Tobramycin",
-    ]
-    MACROLIDES = [
-        "Erythromycin",
-        "Erythromycin/Telithromycin",
-        "Lincosamides",
-        "Lincosamides/Streptogramin",
-        "Macrolides",
-        "Streptogramin",
-    ]
-    RTM = "Other aminoglycoside resistance (non-RMT)"
-    MAC = "Macrolide, lincosamide and/or streptogramin resistance"
-    
-    # amr_data = Data(self.run_type, self.contigs, self.prefix)
 
     def __init__(self, args):
         self.logger =logging.getLogger(__name__) 
@@ -78,10 +48,6 @@ class Collate:
         if the enhanced subclass is in either NONRTM or MACROLIDES then then use the groups specified by Norelle. If it is empty (-) then fall back on the AMRFinder subclass, else report the extended subclass
         """
         gene_id_col = "Gene symbol" if colname != "refseq_protein_accession" else "Accession of closest sequence" # to get the name of drug and the drugclass
-        # print(gene_id_col)
-        print(row[1]['Gene symbol'])
-        # print(colname)
-        # print(reftab[reftab[colname] == row[1][gene_id_col]])
         if reftab[reftab[colname] == row[1][gene_id_col]].empty:
             d = 'Unknown'
             for i in ['genbank_protein_accession','refseq_nucleotide_accession','genbank_nucleotide_accession']:
@@ -90,32 +56,8 @@ class Collate:
                     d= reftab[reftab[i] == row[1][gene_id_col]]['enhanced_subclass'].values[0]
                     break
         else:
-        # try:s
             d = reftab[reftab[colname] == row[1][gene_id_col]]['enhanced_subclass'].values[0]
-        # except:
-        #     d = '-'
-        # if d in self.NONRTM:
-        #     return self.RTM
-        # elif d in self.MACROLIDES:
-        #     return self.MAC
-        # elif d == "-":
-        #     try:
-        #         return reftab[reftab[colname] == row[1]["Gene symbol"]][
-        #             "subclass"
-        #         ].unique()[0]
-        #     except:
-        #         # genbank_protein_accession
-        #         self.logger.warning(f"There appears to be no refseq accession for this so we will try genbank.")
-        #         to_return = 'Unknown'
-        #         # self.logger.info('Will now look for the enhanced subclass for ')
-        #         # print(reftab[reftab[colname] == row[1][gene_id_col]])
-        #         for i in ['genbank_protein_accession','refseq_nucleotide_accession','genbank_nucleotide_accession']:
-        #             if len(reftab[reftab[i] == row[1]["Gene symbol"]]["subclass"].unique()) >= 1:
-        #                 return reftab[reftab[i] == row[1]["Gene symbol"]]["subclass"].unique()[0]
-        #         return to_return
-            
-                
-        # else:
+
         return d
 
     def extract_bifunctional_name(self, protein, reftab):
@@ -187,7 +129,6 @@ class Collate:
         partials = {"Isolate": isolate}
         other = {"Isolate": isolate}
         for row in df.iterrows():
-            # print(row)
             # if the match is good then generate a drugclass dict
             if row[1]["Gene symbol"] == "aac(6')-Ib-cr" and row[1]["Method"] in ["EXACTX", "ALLELEX"]: # This is always a partial - unclear
                 partials = self.setup_dict(drugclass_dict = partials, reftab = reftab, row = row)
@@ -215,25 +156,28 @@ class Collate:
             df[c] = numpy.where(df[c]!="", df[c].str.strip("*") + "^", df[c])
         return df
 
-    def _merge(self, match, partial, pcols, mcols):
+    def _merge(self, match, partial):
 
         df = pandas.DataFrame()
         tmp = match.merge(partial, on = 'Isolate', how = 'outer')
         tmp_cols = self._get_cols(df = tmp)
+            # print(tmp_cols)
         for t in tmp_cols:
-            if t.endswith('_x') or t.endswith('_y'):
+            if t.endswith('_x') or t.endswith('_y'): 
+                # if this is a a col that has appeared in more than one df
                 if df.empty:
                     df = tmp[['Isolate']]
-                c = t.split('_')[0]
-                if c not in list(df.columns):
-                    df[c] = tmp[[f"{c}_x", f"{c}_y"]].apply(lambda x: ','.join(x), axis = 1)
+                c = t.split('_')[0] # strip the _x or _y for col name
+                if c not in list(df.columns): # if this has not already been added - should never happen but best safe than sorry
+                    df[c] = tmp[[f"{c}_x", f"{c}_y"]].apply(lambda x: ','.join(x), axis = 1) # combine the values and join with a comma
             else:
+                # if this is a column that only appears in one df
                 if df.empty:
                     df = tmp[['Isolate', t]]
                 else:
                     df = df.merge(tmp[['Isolate', t]], on = 'Isolate',how = 'outer') 
         
-        return df
+        return df if not df.empty else tmp
 
     def _combine_dfs(self, match, partial, virulence):
         
@@ -242,19 +186,22 @@ class Collate:
         pcols = self._get_cols(df = partial)
         mcols = self._get_cols(df = match)
         vcols = self._get_cols(df = virulence)
-
+        # print(pcols)
         if vcols == [] and pcols == [] and mcols == []:
             return pandas.DataFrame()
 
         if pcols != []:
             partial = self._add_caret(df = partial, cols = pcols)      
         
-        df = self._merge(match = match, partial = partial, mcols = mcols, pcols = pcols)
+        df = self._merge(match = match, partial = partial)
         
-        if isinstance(virulence, pandas.DataFrame) and not df.empty:
+        if not isinstance(virulence, pandas.DataFrame) and not df.empty:
             df = df.merge(virulence, on = 'Isolate', how = 'outer') 
         elif isinstance(virulence, pandas.DataFrame) and df.empty:
-            df = virulence
+            if not virulence.empty:
+                df = virulence
+            else:
+                df = pandas.DataFrame()
 
         return df
 
@@ -268,8 +215,8 @@ class Collate:
             files[f].set_index('Isolate').to_csv(f"{out}", sep = '\t')
         combd = self._combine_dfs(match = match, partial = partial, virulence = virulence)
         combd_out = f"{path}/abritamr.txt" if path != '' else f"abritamr.txt"
-        self.logger.info(f"Saving combined file : {combd_out}")
         if not combd.empty:
+            self.logger.info(f"Saving combined file : {combd_out}")
             combd.set_index('Isolate').to_csv(f"{combd_out}", sep = '\t')
         
         return True
@@ -351,6 +298,7 @@ class Collate:
         self.save_files(path='' if self.run_type == 'batch' else f"{self.prefix}", match = summary_drugs,partial=summary_partial, virulence = virulence)
         
 class MduCollate(Collate):
+    
     def __init__(self, args):
         self.logger =logging.getLogger(__name__) 
         self.logger.setLevel(logging.INFO)
@@ -376,7 +324,7 @@ class MduCollate(Collate):
             "Enterococcus":"Van_Linez_NEG",
             "Other":"Cpase_16S_mcr_NEG"
         }
-
+        self.REPORTING = {"Salmonella enterica":self.mdu_reporting_salmonella}
     def mdu_qc_tab(self):
         self.logger.info(f"Checking the format of the QC file")
         cols = ["ISOLATE", 'SPECIES_EXP', 'SPECIES_OBS', 'TEST_QC']
@@ -475,23 +423,24 @@ class MduCollate(Collate):
     
     def _carbapenem_res_salmo(self, col, gene):
 
-        if "Carbapenemase" in col:
+        if "Carbapenemase" in col: #not KPC
             return gene
         return ''
 
     def _azi_res_salmo(self, col, gene):
 
-        if 'Azithromycin' in  col:
+        if 'Azithromycin' in  col or col in ['Macrolide', 'Lincosamide/Macrolide/Streptogramin']:
             return gene
         return ''
 
     def _gentamicin_res_salm(self, col, gene):
-        if 'Gentamicin' in col:
+        if col in ['Gentamicin', 'Aminoglycosides (Ribosomal methyltransferase)']: #change
             return gene
         return ''
     
     def _kanamycin_res_salm(self, col, gene):
-        if 'Kanamycin' in col:
+    
+        if col in ['Kanamycin','Aminoglycosides (Ribosomal methyltransferase)']:
             return gene
         return ''
     
@@ -529,9 +478,9 @@ class MduCollate(Collate):
             return gene
         return ''
     
-    def _trim_sulpha_salmo(self, col, gene):
+    def _trim_sulpha_salmo(self, trim_gene, sul_gene):
 
-        pass
+        return f"{trim_gene},{sul_gene}"
 
     def _rmt_res_salmo(self, col, gene):
 
@@ -560,8 +509,7 @@ class MduCollate(Collate):
         
         all_genes = self.get_all_genes(row)
         all_genes = [a for a in all_genes if a != row[1]['Isolate'] and a != '']
-        # print(all_genes)
-        
+                
         isodict = row[1].to_dict()
         item_code = self.assign_itemcode(row[1]['Isolate'], mduidreg)
         md = self.assign_mduid(row[1]['Isolate'], mduidreg)
@@ -570,7 +518,7 @@ class MduCollate(Collate):
             "Ampicillin" : self._ampicillin_res_sal,
             "Cefotaxime (ESBL)":self._cefo_esbl_res_sal,
             "Cefotaxime (AmpC)":self._cefo_ampc_res_sal,
-            "Carbapenem" :self._carbapenem_res_salmo,
+            "Meropenem" :self._carbapenem_res_salmo,
             "Azithromycin":self._azi_res_salmo,
             "Gentamicin":self._gentamicin_res_salm,
             "Kanamycin":self._kanamycin_res_salm,
@@ -601,42 +549,57 @@ class MduCollate(Collate):
             "Spectinomycin":[],
             "Ciprofloxacin":[],
             "Azithromycin":[],
-            "Carbapenem":[],
+            "Meropenem":[],
             "Chloramphenicol":[],
             "Aminoglycosides (Ribosomal methyltransferase)":[],"Colistin":[], "Other":[]
         }
         # group drugs
         # NEED TO ADD in TRIM-SULPHA combo
         for ab in abx:
-            for i in isodict:
-                if i != "Isolate" and ab != 'Trim-Sulpha':
-                    # print(isodict[i])
-                    g = abx[ab](col = i, gene = isodict[i])
-                    # print(g)
-                    if g != '':
-                        gene_list = g.split(',')
-                        for gene in gene_list:
-                            tmp_results[ab].append(gene)
-                            all_genes.remove(gene)
-        
+            # if ab == 'Trim-Sulpha':
+            #     if 'Trimethoprim' in isodict and 'Sulfathiazole' in isodict:
+            #         g = abx[ab](trim_col = isodict['Trimethoprim'], sul_col = isodict['Sulfathiazole'])
+            #         if g != '':
+            #             gene_list = g.split(',')
+            #             for gene in gene_list:
+            #                 tmp_results[ab].append(gene)
+            if ab != 'Trim-Sulpha':
+                for i in isodict:
+                    if i != "Isolate":
+                        # print(isodict[i])
+                        g = abx[ab](col = i, gene = isodict[i])
+                        # print(g)
+                        if g != '':
+                            gene_list = g.split(',')
+                            for gene in gene_list:
+                                tmp_results[ab].append(gene)
+                                if gene in all_genes:
+                                    all_genes.remove(gene)
+            
         # interprete results
         tmp_results['Other'] = all_genes
         results = {'Isolate': row[1]['Isolate'], 'MDU Sample ID':md, 'Item code': item_code}
-
+        # for Trim-Sulpha
+        
+        if tmp_results['Trimethoprim'] != [] and tmp_results['Sulfathiazole'] != []:
+            tmp_results['Trim-Sulpha'] = list(set(tmp_results['Trimethoprim']).union(tmp_results['Sulfathiazole']))
+        # print(tmp_results)
         for res in tmp_results:
-            results[f"{res} - ResMech"] = ';'.join(tmp_results[res])                
-            if res == 'Ciprofloxacin':
+            results[f"{res} - ResMech"] = ';'.join(tmp_results[res]) if tmp_results[res] != [] else "None detected"              
+            if res in ["Aminoglycosides (Ribosomal methyltransferase)","Colistin", "Other"]:
+                results[f"{res} - Interpretation"] = ''
+            elif res == 'Ciprofloxacin':
                 if tmp_results[res] == []:
-                    results[f"{res} - Interpret"] = 'Susceptible'
+                    results[f"{res} - Interpretation"] = 'Susceptible'
                 elif len(tmp_results[res]) == 1:
-                    results[f"{res} - Interpret"] = 'Intermediate'
+                    results[f"{res} - Interpretation"] = 'Intermediate'
                 else:
-                    results[f"{res} - Interpret"] = 'Resistant'
+                    results[f"{res} - Interpretation"] = 'Resistant'
             elif res not in ["Aminoglycosides (Ribosomal methyltransferase)","Colistin", "Other"]:
                 if tmp_results[res] == []:
-                    results[f"{res} - Interpret"] = 'Susceptible'
+                    results[f"{res} - Interpretation"] = 'Susceptible'
                 else:
-                    results[f"{res} - Interpret"] = 'Resistant'
+                    results[f"{res} - Interpretation"] = 'Resistant'
         
         return results
 
@@ -720,8 +683,8 @@ class MduCollate(Collate):
                         else:
                             genes_not_reported.extend(genes)
 
-                else:
-                    genes_not_reported.extend(genes)
+                    else:
+                        genes_not_reported.extend(genes)
         if genes_reported == []:
             genes_reported = [self.none_replacement_code(genus= genus)] if neg_code else ''
         if genes_not_reported == []:
@@ -731,7 +694,7 @@ class MduCollate(Collate):
         self.logger.info(f"{row[1]['Isolate']} has {len(genes_reported)} reportable genes.")
         return genes_reported, genes_not_reported
 
-    def mdu_reporting_salmonella(self, match):
+    def mdu_reporting_salmonella(self, match, isolates):
 
         self.logger.info(f"Applying MDU business logic for interpretation of  Salmonella AST")
         cols = ["Isolate", "MDU Sample ID", "Item code", 
@@ -740,17 +703,25 @@ class MduCollate(Collate):
         "Cefotaxime (AmpC) - ResMech","Cefotaxime (AmpC) - Interpretation",
         "Tetracycline - ResMech","Tetracycline - Interpretation",
         "Gentamicin - ResMech","Gentamicin - Interpretation",
+        "Kanamycin - ResMech",	"Kanamycin - Interpretation",
+        "Streptomycin - ResMech",	"Streptomycin - Interpretation",
         "Sulfathiazole - ResMech","Sulfathiazole - Interpretation",
         "Trimethoprim - ResMech","Trimethoprim - Interpretation",
+        "Trim-Sulpha - ResMech",	"Trim-Sulpha - Interpretation",
+        "Chloramphenicol - ResMech",	"Chloramphenicol - Interpretation",
         "Ciprofloxacin - ResMech","Ciprofloxacin - Interpretation",
+        "Meropenem - ResMech",	"Meropenem - Interpretation",
         "Azithromycin - ResMech","Azithromycin - Interpretation",
-        "Aminoglycosides (Ribosomal methyltransferase) - ResMech","Colistin - ResMech", "Other - ResMech"]
+        "Aminoglycosides (Ribosomal methyltransferase) - ResMech",
+        "Aminoglycosides (Ribosomal methyltransferase) - Interpretation",
+        "Colistin - ResMech",
+        "Colistin - Interpretation", 
+        "Other - ResMech",
+        "Other - Interpretation"]
         # select passed Salmonella
-        qc = self.mdu_qc_tab()
-        qc = qc[(qc['SPECIES_OBS'] == 'Salmonella enterica') & (qc['TEST_QC'] == 'PASS')]
-
+        
         df = pandas.read_csv(match, sep = '\t')
-        df = df[df['Isolate'].isin(list(qc['ISOLATE']))]
+        df = df[df['Isolate'].isin(isolates)]
         result_df = pandas.DataFrame()
         df = df.fillna('')
         for row in df.iterrows():
@@ -762,6 +733,14 @@ class MduCollate(Collate):
                 result_df = result_df.append(tmpdf)
         return result_df[cols]
 
+    def _extract_plus_isolates(self,species):
+
+        qc = self.mdu_qc_tab()
+        qc = qc[(qc['SPECIES_OBS'] == species) & (qc['TEST_QC'] == 'PASS')]
+
+        return list(qc["ISOLATE"])
+
+
     def mdu_reporting_general(self, match, neg_code = True):
 
         self.logger.info(f"Applying MDU business logic {'matches' if neg_code else 'partials'}.")
@@ -819,20 +798,30 @@ class MduCollate(Collate):
         writer.close()
 
     def save_spreadsheet_interpreted(self, results):
+        sheets = {"Salmonella enterica":"MMS184-01"}
         self.logger.info(f"Saving MMS184")
         writer = pandas.ExcelWriter(f"{self.runid}_MMS184.xlsx", engine = "xlsxwriter")
-        results.to_excel(writer, sheet_name = "MMS184")
+        for result in results:
+            result[1].to_excel(writer, sheet_name = sheets[result[0]], index = False)
         writer.close()
 
     def run(self):
-        if self.sop == 'general':
+        if self.sop == 'general' and pathlib.Path(self.partials).exists():
             passed_match_df = self.mdu_reporting_general(match=self.match)
             passed_partials_df = self.mdu_reporting_general(match = self.partials)
             self.save_spreadsheet_general(
                 passed_match_df,
                 passed_partials_df
             )
-        elif self.sop == 'salmonella':
-
-            salmo_df = self.mdu_reporting_salmonella(match = self.match)
-            self.save_spreadsheet_interpreted(results = salmo_df)
+        elif self.sop == 'plus':
+            dfs = []
+            for r in self.REPORTING:
+                self.logger.info(f"Checking if there are any {r} in this run.")
+                isolates = self._extract_plus_isolates(species = r)
+                if isolates != []:
+                    self.logger.info(f"There are {len(isolates)} {r} in this run.")
+                    plus_df = self.mdu_reporting_salmonella(match = self.match, isolates=isolates)
+                    dfs.append((r,plus_df))
+                else:
+                    self.logger.info(f"There are no {r} in this run. Collation will be skipped.")
+            self.save_spreadsheet_interpreted(results = dfs)
